import { decodeComponentId } from '@previewjs/api';
import { createChromelessWorkspace } from '@previewjs/chromeless';
import { globby } from 'globby';

async function generateScreenshots({
  page,
  frameworkPlugins,
  filePathPattern,
  cwd = process.cwd(),
  suppressErrors = false,
  generateScreenshotPath,
  onScreenshot
}) {
  const workspace = await createChromelessWorkspace({
    frameworkPlugins,
    rootDirPath: cwd
  });
  const preview = await workspace.preview.start(page);
  const filePaths = await globby(filePathPattern, {
    gitignore: true,
    ignore: ["**/node_modules/**"],
    cwd,
    followSymbolicLinks: false
  });
  const { components } = await workspace.detectComponents({
    filePaths
  });
  for (const component of components) {
    const { filePath, name } = decodeComponentId(component.componentId);
    try {
      await preview.show(component.componentId);
      const screenshotPath = generateScreenshotPath({ filePath, name });
      await preview.iframe.takeScreenshot(screenshotPath);
      if (onScreenshot) {
        onScreenshot({ filePath, name, screenshotPath });
      }
    } catch (e) {
      const messsage = `Unable to generate screenshot for ${component.componentId}`;
      if (suppressErrors) {
        console.warn(`${messsage}:
${e.message}
`);
      } else {
        throw new Error(messsage, { cause: e });
      }
    }
  }
  await preview.stop();
  await workspace.dispose();
}

export { generateScreenshots };
